<?

function sort_by_name_function($a, $b) {
 return strcmp($a['name'], $b['name']);
}

class Bundle extends NodeObject {
 var $bundle_id;
 
 var $_builds;
 
 var $_tables;

 function getTType() {
 return T_BUNDLE;
 }

 function _setID($id) {
 $this->bundle_id = $id;
 }

 function getID() {
 return $this->bundle_id;
 }

 function _getNodeTable() {
 return "bundle";
 }

 function getValueFields() {
 return array( 'name' => 'text',
 'description' => 'textarea',
 'password' => 'text',
 'globalprefix' => 'text',
 'globalid' => 'int',
 'isimported' => 'int',
 'keepoldbuilds' => 'int',
 'enabledownload' => 'int',
 'creator' => false,
 'editor' => false,
 'cdate' => false,
 'edate' => false,
 'importfolder_id' => 'int',
 'updateurl' => 'text',
 'source' => 'text',
 'importgroup_id' => 'int' );
 }

 function getStatusMessage() {
 
 $build = &$this->getLatestBuild();
 if( $build === NULL ) {
 return array( 'message' => i18n('No Build'), 'status' => 'NOBUILD' );
 }
 return $build->getStatusMessage(true);
 }

 function &getLatestBuild() {
 $sql = "SELECT id FROM bundlebuild WHERE bundle_id = '".sqlf_escaped($this->getID())."' ORDER BY date desc limit 1";
 $rs = query( $sql );
 if(count($rs['arr']) != 1) {
 return null;
 }
 $build = & NodeFactory::getStaticInstanceOf( T_BUNDLE_BUILD, $rs['arr'][0]['id'] );
 return $build;
 }

 function getLatestDownloadableBuild() {
 $sql = "SELECT id FROM bundlebuild WHERE bundle_id = '".sqlf_escaped($this->getID())."' AND statuscode = 0 ORDER BY date desc";
 $rs = query( $sql );
 foreach( $rs['arr'] as $row ) {
 $build = &NodeFactory::getStaticInstanceOf( T_BUNDLE_BUILD, $rs['arr'][0]['id'] );
 if( $build->isDownloadable() ) {
 return $build;
 }
 }
 return null;
 }

 function createObjects($ttype, &$arr) {
 $ret = array( );
 foreach( $arr as $row ) {

 
 $obj = NodeFactory::getStaticInstanceOf( $ttype, $row['id'] );
 if( !$obj->isInitialized() ) {
 $obj->initFromResultSet( $row, '', true );
 }
 $ret[] = $obj;
 }
 return $ret;
 }

 



 function &getImportBundleByGlobalId( $globalprefix, $globalid ) {
 $sql = "SELECT id FROM bundle WHERE globalprefix = '".sqlf_escaped($globalprefix)."' AND globalid = '".sqlf_escaped($globalid)."' AND isimported = 1";
 $rs = query( $sql );
 $id = $rs['arr'][0]['id'];
 if( !$id ) {
 return NULL;
 }
 $b = NodeFactory::getStaticInstanceOf( T_BUNDLE, $id );
 return $b;
 }

 function &getBuilds($onlyfinished = false) {
 
 if ($onlyfinished) {
 if ($this->getFieldValue("isimported")) {
 $join = "LEFT JOIN bundleimport ON bundlebuild.id = bundleimport.bundlebuild_id";
 $restrict = "AND bundleimport.date != 0";
 } else {
 $restrict = "AND bundlebuild.statuscode = 0";
 }
 }
 $sql = "SELECT bundlebuild.* FROM bundlebuild $join WHERE bundlebuild.bundle_id = '".sqlf_escaped($this->getID())."' $restrict ORDER BY bundlebuild.id DESC";
 $rs = query( $sql );
 return $this->createObjects( T_BUNDLE_BUILD, $rs['arr'] );
 }

 



 function &getBuildByDate( $date ) {
 $sql = "SELECT id FROM bundlebuild WHERE bundle_id = '".sqlf_escaped($this->getID())."' AND date = '".sqlf_escaped($date)."'";
 $rs = query( $sql );
 $id = $rs['arr'][0]['id'];
 if(!$id) {
 return NULL;
 }
 $build = NodeFactory::getStaticInstanceof( T_BUNDLE_BUILD, $id );
 return $build;
 }

 function onBeforeInsertObject() {
 global $USER_ID, $TIME;
 if( !$this->getFieldValue('isimported') && !$this->getFieldValue('globalprefix') ) {
 $this->setFieldValue('globalprefix', get_globalprefix());
 $this->setFieldValue('globalid', create_globalid());
 }
 $this->setFieldValue('creator', $USER_ID);
 $this->setFieldValue('editor', $USER_ID);
 $this->setFieldValue('cdate', $TIME);
 $this->setFieldValue('edate', $TIME);
 }

 function onInsertObject() {
 global $USER_ID;
 $isImp = $this->getFieldValue('isimported');
 $ttype = $isImp ? T_BUNDLE_IMPORT_ELEMENT : T_BUNDLE_EXPORT_ELEMENT;

 
 perm_grant_all($ttype, $this->getID(), false, "11110000000000000000000000000000");
 }

 function onBeforeUpdateObject() {
 global $USER_ID, $TIME;
 $this->setFieldValue('editor', $USER_ID);
 $this->setFieldValue('edate', $TIME);
 }

 function onBeforeDeleteObject() {
 
 $builds = &$this->getBuilds();
 foreach( $builds as $build ) {
 $build->delete();
 }

 
 $isimported = $this->getFieldValue('isimported');
 $ttype = $isimported ? T_BUNDLE_IMPORT_ELEMENT : T_BUNDLE_EXPORT_ELEMENT;
 $sql = "DELETE FROM perm WHERE o_type = ".sqlf_tl_num($ttype)." AND o_id = ".sqlf_tl_num($this->getID());
 query($sql);
 }



 function toXML() {
 $sourcehost = $this->getFieldValue('source');
 $globalprefix = $this->getFieldValue('globalprefix');
 if( $globalprefix == get_globalprefix() ) {
 $sourcehost = get_sourcehost();
 }
 $str = 
 "<bundle>\n" .
 "  <bundleinfo>\n" .
 "    <name><![CDATA[".$this->getFieldValue('name')."]]></name>\n" .
 "    <description><![CDATA[".$this->getFieldValue('description')."]]></description>\n" .
 "    <globalid>" . $this->getFieldValue('globalid') . "</globalid>\n" .
 "    <globalprefix>". $globalprefix . "</globalprefix>\n" .
 "    <sourcehost><![CDATA[$sourcehost]]></sourcehost>\n" .
 "  </bundleinfo>\n" .
 "  <builds>\n";
 
 $builds = &$this->getBuilds();
 foreach( $builds as $build ) {
 $str .= $build->toXML();
 }

 $str .=
 "  </builds>\n" .
 "  <objects>\n";

 if ($build) {
 $objs = &$build->getContainedObjects(false);
 foreach( $objs as $obj ) {
 $str .= $obj->toXML();
 }
 }

 $str .= 
 "  </objects>\n" .
 "</bundle>\n";
 return $str;
 }



 







 function& parseXML($xml, $parseObjects = true) {
 $p = xml_parser_create();
 if(!xml_parse_into_struct($p, $xml, $vals, $index)) {
 return NULL;
 }
 xml_parser_free($p);

 
 $i = 0;
 $bundle = new Bundle();
 $attrs = array( 'NAME' => 'name',
 'DESCRIPTION' => 'description',
 'GLOBALID' => 'globalid',
 'GLOBALPREFIX' => 'globalprefix',
 'SOURCEHOST' => 'source', );
 for($i = 0 ; $i < count( $vals ) ; $i++ ) {
 if( $vals[$i]['tag'] == 'BUNDLE' ) {
 $started = TRUE;
 continue;
 }
 if( $vals[$i]['level'] < 2 ) {
 continue;
 }

 $attr = $attrs[$vals[$i]['tag']];
 if( $attr ) {
 
 $bundle->setFieldValue( $attr, $vals[$i]['value'] );
 }

 if( $vals[$i]['tag'] == 'BUILDS' ) {
 $i++;
 while($vals[$i]['level'] > 2 || ($vals[$i]['tag'] == 'BUILDS' && $vals[$i]['type'] != 'close')) {
 unset($bundlebuild);
 $bundlebuild = &BundleBuild::parseXML( &$vals, &$i );
 if( $bundlebuild !== null ) {
 $bundle->_builds[] = $bundlebuild;
 }
 $i++;
 }

 }
 if( $vals[$i]['tag'] == 'OBJECTS' ) {
 $i++;
 while($parseObjects && ($vals[$i]['level'] > 2 || ($vals[$i]['tag'] == 'OBJECTS' && $vals[$i]['type'] != 'close'))) {
 unset($obj);
 if ($vals[$i]['tag'] == 'OBJECT' && $vals[$i]['type'] != 'close') {
 $obj = &BundleContainedObject::parseXML( &$vals, &$i );
 }
 if( $obj !== null ) {
 $bundle->_objects[] = $obj;
 }
 $i++;
 }

 }
 if( $vals[$i]['tag'] == 'TABLES' ) {
 $i++;
 while($vals[$i]['level'] > 2 || ($vals[$i]['tag'] == 'TABLES' && $vals[$i]['type'] != 'close')) {
 unset($table);
 $table = &Bundle::parseTableXML( &$vals, &$i );
 if( $table !== null ) {
 $bundle->_tables[] = $table;
 }

 $i++;
 }

 }
 }
 return $bundle;
 }

 function &getCurrentBundleImport() {
 $sql = "SELECT bundleimport.id FROM bundleimport INNER JOIN bundlebuild ON bundlebuild.id = bundleimport.bundlebuild_id WHERE bundlebuild.bundle_id = ".sqlf_tl_num($this->getID())." AND bundleimport.date = 0";
 $rs = query( $sql );
 if (count($rs["arr"]) < 1) {
 return null;
 }
 
 return NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORT, $rs['arr'][0]['id'] );
 }

 function &getLatestBundleImport() {
 $sql = "SELECT bundleimport.id as bundleimportid, bundleimport.date AS importdate FROM bundleimport JOIN (bundlebuild) ON (bundleimport.bundlebuild_id=bundlebuild.id) WHERE bundlebuild.bundle_id='" . sqlf_escaped($this->getID()) . "' ORDER BY importdate != 0, importdate DESC;";
 $rs = query( $sql );
 if (count($rs["arr"]) < 1) {
 return null;
 }
 return NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORT, $rs["arr"][0]["bundleimportid"] );
 }

 function &getInstalledBundleBuild() {
 $sql = "SELECT bundlebuild.id as bundlebuildid FROM bundleimport JOIN (bundlebuild) ON (bundleimport.bundlebuild_id=bundlebuild.id) WHERE bundlebuild.bundle_id='" . sqlf_escaped($this->getID()) . "' AND bundleimport.date != 0 ORDER BY bundleimport.date DESC LIMIT 1;";
 $rs = query( $sql );
 if (count($rs["arr"]) < 1) {
 return null;
 }
 return NodeFactory::getStaticInstanceOf( T_BUNDLE_BUILD, $rs["arr"][0]["bundlebuildid"] );
 }


 function& parseTableXML(&$vals, &$i) {
 if( $vals[$i]['tag'] != 'TABLE' ) {
 return null;
 }
 $lvl = $vals[$i]['level'];
 $table = array();
 for( ; $i < count( $vals ) ; $i++ ) {
 if( $vals[$i]['level'] <= $lvl && $vals[$i]['type'] == 'close' ) {
 break;
 }
 $tag = $vals[$i]['tag'];
 $val = $vals[$i]['value'];
 $table[strtolower($tag)] = $val;
 }
 return $table;
 }

 function getUpdateUrl($usePassword) {
 $url = $this->getFieldValue("updateurl");
 if ($usePassword) {
 $url .= "&pw=".$this->getFieldValue("password");
 }
 return $url;
 }


 function &checkForUpdates() {
 $updateUrl = $this->getUpdateUrl(false);
 $updateBuild = &Bundle::getLatestBuildFromUpdate($updateUrl);
 
 
 $newestBuild = &$this->getLatestBuild();
 if ($updateBuild) {
 if ($updateBuild->getFieldValue('date') > $newestBuild->getFieldValue('date')) {
 $updateBuild->setFieldValue('bundle_id', $this->getID());
 $updateBuild->save();
 return $updateBuild;
 }
 }
 return false;
 }

 function getLatestBuildFromUpdate($updateUrl) {
 $contents = file_get_contents($updateUrl);
 $updatedBundle = Bundle::parseXML($contents, false);
 $builds = &$updatedBundle->_builds;
 $updateBuild = null;
 $updateDate = 0;
 
 foreach ($builds as $build) {
 if ($build->getFieldValue('downloadurl')) {
 if ($build->getFieldValue('date') > $updateDate) {
 $updateBuild =& $build;
 $updateDate = $build->getFieldValue('date');
 }
 unset($build);
 }
 }
 return $updateBuild;
 }

 



 function getDownloadURL() {
 global $STAG_PREFIX, $CN_LOCAL_PATH, $CN_LOCAL_SERVER, $_SERVER;
 
 $isHttps = $_SERVER["HTTPS"] != null;
 $port = $_SERVER["SERVER_PORT"];
 if ($port == 80 && ! $isHttps) {
 $port = "";
 } else if ($port == 443 && $isHttps) {
 $port = "";
 } else {
 $port = ":" . $port;
 }
 $protocol = $isHttps ? "https" : "http";
 return $protocol . '://' . $_SERVER['SERVER_NAME'] . $port . $STAG_PREFIX . '?do=1018&bundle_id='.$this->getID();
 }

 


 function isDuplicateName($name, $bundle_id=0, $export=true) {
 $sql = "SELECT COUNT(*) AS count FROM bundle WHERE name = '" . sqlslashes($name) . "' AND isimported = ";
 if ($export) {
 $sql .= "0";
 } else {
 $sql .= "1";
 }
 if ($bundle_id != 0) {
 $sql .=" AND id != '" . sqlslashes($bundle_id) . "'";
 }
 $rs = query($sql);
 if ($rs["arr"][0]["count"] > 0) {
 return true;
 } else {
 return false;
 }
 }

 



 function userMayEdit() {
 global $USER_ID;
 $isImp = $this->getFieldValue('isimported');
 return perm($USER_ID, T_BUNDLE, '', 3) || perm($USER_ID, $isImp ? T_BUNDLE_IMPORT_ELEMENT : T_BUNDLE_EXPORT_ELEMENT, $this->getID(), 1);
 }

 



 function userMayCreateBundle() {
 global $USER_ID;
 $isImp = $this->getFieldValue('isimported');
 return perm($USER_ID, T_BUNDLE, '', $isImp ? 6 : 5) || perm($USER_ID, $isImp ? T_BUNDLE_IMPORT_ELEMENT : T_BUNDLE_EXPORT_ELEMENT, $this->getID(), 2);
 }

 



 function createCopy() {
 
 $oldname = $this->getFieldValue("name");
 $num = 0;

 $isImp = $this->getFieldValue('isimported');
 $ttype = $isImp ? T_BUNDLE_IMPORT_ELEMENT : T_BUNDLE_EXPORT_ELEMENT;

 do {
 $num++;
 $newname = i18n( "Copy {0} of {1}" , 0, $num, $oldname);
 $unique = !$this->isDuplicateName($newname, 0, !$isImp);
 } while(!$unique);

 $bundle = new Bundle();
 $bundle->setFieldValue( 'isimported', $isImp );
 $bundle->setFieldValue( 'name', $newname );
 $bundle->setFieldValue( 'description', $this->getFieldValue('description') );
 $bundle->setFieldValue( 'enabledownload', $this->getFieldValue('enabledownload') );
 $bundle->setFieldValue( 'keepoldbuilds', $this->getFieldValue('keepoldbuilds') );
 $bundle->setFieldValue( 'password', $this->getFieldValue('password') );
 $bundle->save();

 
 $latestbuild = $this->getLatestBuild();
 if ($latestbuild) {
 $latestbuild->createCopy($bundle->getID());
 }

 
 duplicate_perm($ttype, $this->getID(), $ttype, $bundle->getID());

 return $bundle;
 }
}

class BundleContainedObject extends NodeObject {
 var $bundlecontainedobject_id;

 function _getNodeTable() {
 return "bundlecontainedobject";
 }

 function _setID($id) {
 $this->bundlecontainedobject_id = $id;
 }
 function getID() {
 return $this->bundlecontainedobject_id;
 }

 function &getActualObject() {
 $type = $this->getFieldValue( 'obj_type' );
 $id = $this->getFieldValue( 'obj_id' );
 $channelId = $this->getFieldValue( 'channel_id' );
 if ($channelId) {
 $id = "$id.$channelId";
 }
 return NodeFactory::getStaticInstanceOf( $type, $id );
 }

 function getValueFields() {
 return array( 'bundle_id' => false,
 'bundlebuild_id' => false,
 'obj_type' => 'int',
 'obj_id' => 'int',
 'channel_id' => 'int',
 'referrer_obj_id' => 'int',
 'referrer_obj_type' => 'int',
 'autoadded' => 'int',
 'excluded' => 'int'
 );
 }


 function toXML() {
 global $T_TYPE;
 $o = &$this->getActualObject();
 if ($o->isInitialized()) {
 $str =
 "<object>\n" .
 "  <ttype>" . $o->getTType() . "</ttype>\n" .
 "  <humantype>" . $T_TYPE[$o->getTType()]['name'] . "</humantype>\n" .
 "  <humanname><![CDATA[" . $o->getDisplayName() . "]]></humanname>\n" .
 "</object>\n";
 } 
 return $str;
 }

 function& parseXML( &$vals, &$i ) {
 $valsIX = $vals[$i];
 if( $valsIX['tag'] != 'OBJECT' ) {
 return null;
 }
 $lvl = $valsIX['level'];
 $obj = array();
 set_time_limit(60);
 $count = count($vals);
 for( ; $i < $count ; $i++ ) {
 $valI = $vals[$i];
 if( $valI['level'] <= $lvl && $valI['type'] == 'close' ) {
 break;
 }
 $tag = $valI['tag'];
 $val = $valI['value'];
 $obj[strtolower($tag)] = $val;
 }

 return $obj;
 }

}

class BundleBuild extends NodeObject {
 var $bundlebuild_id;

 var $_objects;

 function _getNodeTable() {
 return "bundlebuild";
 }

 function _setID($id) {
 $this->bundlebuild_id = $id;
 }
 function getID() {
 return $this->bundlebuild_id;
 }

 function getStatusMessage($islast=false) {
 global $STAG, $USER_ID;
 
 $statuscode = $this->getFieldValue('statuscode');
 if( !$this->getFieldValue('filename') ) {
 if( $statuscode == 2 ) {
 return array( 'message' => i18n('Error during build.'), 'status' => 'ERROR' );
 }
 if( $statuscode == 3 ) {
 return array( 'message' => i18n('sn_process_canceled'), 'status' => 'ERROR' );
 }
 if ($statuscode == 5) {
 return array( 'message' => i18n('Need confirmation'), 'status' => 'NOTYETBUILT');
 }
 node_class("codeparser");
 $javaParser =& CodeParser::getJavaParser();
 $status = $javaParser->statusBundle( $this->getID() );
 
 
 $s = $status['status'];
 if($s == 'QUEUED') {
 return array( 'message' => i18n('Queued {0} Position', 0, $status['queuePosition']), 'status' => 'QUEUED ' . $status['queuePosition'] );
 } else if($s == 'INPROGRESS') {
 $cancel = "";
 if (perm( $USER_ID, T_BUNDLE_EXPORT_ELEMENT, $this->getFieldValue("bundle_id"), 2 ) || perm( $USER_ID, T_BUNDLE, '', 5 )) {
 $cancel = " / <a href=\"$STAG&do=1010&cmd=cancel&type=export&bundlebuildid=" . $this->getID() . "\">" . i18n('Cancel') . "</a>";
 }
 return array( 'message' => i18n('In Progress ({0} %)', 0, $status['progressPercent']), 'status' => 'INPROGRESS ' . $status['progressPercent'], 'html' => i18n('In Progress ({0} %)', 0, $status['progressPercent']) . $cancel, 'refresh' => true);
 }
 
 if($statuscode == -1) {
 return array( 'message' => i18n('Error during build.'), 'status' => 'ERROR');
 }
 return array( 'message' => i18n('Not yet built.'), 'status' => 'NOTYETBUILT' );
 }
 
 $warnings = $this->getWarningCount();
 if( $warnings ) {
 if( $islast ) {
 return array( 'html' => "<a style=\"color: #FF8827;\" href=\"$STAG&do=1020&bundle_id=" . $this->getFieldValue('bundle_id') . "\">" . i18n('{0} referenced objects', 0, $warnings) . "</a>",
 'message' => i18n('{0} Warnings', 0, $warnings),
 'status' => 'WARNINGS ' . $warnings );
 } else {
 return array( 'message' => i18n( 'Produced warnings.' ),
 'status' => 'PRODUCEDWARNINGS' );
 }
 } else {
 return array( 'message' => i18n('Done'),
 'status' => 'DONE' );
 }
 }

 



 function getWarningCount() {
 $sql = "SELECT COUNT(*) as c FROM bundlecontainedobject WHERE referrer_obj_id != 0 AND accepted = 0 AND bundle_id = ".sqlf_tl_num($this->getFieldValue('bundle_id'))." AND bundlebuild_id = '".sqlf_escaped($this->getID())."'";
 $rs = query( $sql );
 $warnings = $rs['arr'][0]['c'];
 return $warnings;
 }

 function getValueFields() {
 return array( 'bundle_id' => false,
 'changelog' => 'textarea',
 'date' => 'int',
 'filename' => 'text',
 'statuscode' => 'int',
 );
 }

 function &getBundle() {
 $bundle = &NodeFactory::getStaticInstanceOf(T_BUNDLE, $this->getFieldValue('bundle_id'));
 return $bundle;
 }

 function getDownloadURL() {
 global $STAG_PREFIX, $CN_LOCAL_PATH, $CN_LOCAL_SERVER, $_SERVER;
 $filename = $this->getFieldValue('filename');
 if( !$this->isDownloadable()) {
 return NULL;
 }
 $bundle = &$this->getBundle();
 
 $isHttps = $_SERVER["HTTPS"] != null;
 $port = $_SERVER["SERVER_PORT"];
 if ($port == 80 && ! $isHttps) {
 $port = "";
 } else if ($port == 443 && $isHttps) {
 $port = "";
 } else {
 $port = ":" . $port;
 }
 $protocol = $isHttps ? "https" : "http";
 return $protocol . '://' . $_SERVER['SERVER_NAME'] . $port . $STAG_PREFIX . '?do=1018&bundle_id='.$this->getFieldValue('bundle_id').'&pw='.$bundle->getFieldValue('password').'&build_id='.$this->getID().'&'.$this->getFieldValue('filename');
 }

 function isDownloadable() {
 $filename = $this->getFieldValue('filename');
 if( !$filename ) {
 return false;
 }
 if ($this->getFileSize() === 0) {
 return false;
 }
 $statuscode = $this->getFieldValue('statuscode');
 
 return $statuscode == 0;
 }

 function revalidateStatus() {
 $statuscode = $this->getFieldValue('statuscode');
 if( $statuscode != 1 ) {
 return;
 }
 
 $warnings = $this->getWarningCount();
 if( $warnings == 0 ) {
 $this->setFieldValue('statuscode', 0);
 $this->save();
 } else {
 
 }
 }

 function toXML() {
 $str = 
 "<build>\n" .
 "  <builddate>" . $this->getFieldValue( 'date' ) . "</builddate>\n" .
 "  <humanbuilddate>" . db2date($this->getFieldValue( 'date' )) . "</humanbuilddate>\n" .
 "  <changelog><![CDATA[" . $this->getFieldValue( 'changelog' ) . "]]></changelog>\n" .
 "  <downloadurl>" . htmlspecialchars($this->getDownloadURL()) . "</downloadurl>\n" .
 "</build>\n";
 return $str;
 }

 function& parseXML( &$vals, &$i ) {
 if( $vals[$i]['tag'] != 'BUILD' ) {
 return null;
 }
 $lvl = $vals[$i]['level'];
 $build = new BundleBuild();
 for( ; $i < count( $vals ) ; $i++ ) {
 if( $vals[$i]['level'] <= $lvl && $vals[$i]['type'] == 'close' ) {
 break;
 }
 $tag = $vals[$i]['tag'];
 $val = $vals[$i]['value'];
 if( $tag == 'BUILDDATE' ) {
 $build->setFieldValue( 'date', $val );
 } else if( $tag == 'CHANGELOG' ) {
 $build->setFieldValue( 'changelog', $val );
 } else if( $tag == 'DOWNLOADURL' ) {
 $build->setFieldValue( 'downloadurl', $val );
 }
 }
 return $build;
 }

 function getBundleFileNameForDate( $date ) {
 return "import_" . $date . "_".getmypid().".zip";
 }

 function getBundleFileName() {
 if (!$this->getFieldValue('filename')) {
 $this->setFieldValue('filename', BundleBuild::getBundleFileNameForDate( $this->getFieldValue( 'date' )));
 }
 return $this->getFieldValue('filename');
 }

 function getBundleFilePath($date = null) {
 global $FILEPATH;
 
 if( $date === null ) {
 $date = $this->getFieldValue( 'date' );
 }
 $filename = BundleBuild::getBundlePath() . BundleBuild::getBundleFileName($date);
 return $filename;
 }

 function getBundlePath() {
 global $FILEPATH;
 return "$FILEPATH/system/bundles/";
 }

 function &downloadFile($showprogress = false) {
 global $FILEPATH;


 if( $showprogress ) {
 $progress = new Progress();
 $progress->init();
 }


 
 $this->setFieldValue('filename', $this->getBundleFileName());
 $filename = $this->getBundleFilePath();
 $outfile = fopen( $filename, 'w' );
 $handle = fopen( $this->getFieldValue('downloadurl'), 'rb' );

 $bufsize = 1024 * 1024; 
 $meta = stream_get_meta_data( $handle );
 
 $headers = $meta['wrapper_data'];
 $size = -1;
 foreach( $headers as $header ) {
 $h = split('\\s*:\\s*', $header);
 if( $h[0] == 'Content-Length' ) {
 $size = $h[1];
 }
 }
 $downloaded = 0;
 $lastpercent = 0;
 sleep(5);
 if( $showprogress ) {
 $progress->start_size($size);
 }

 while(!feof($handle) && ($str = fread( $handle, $bufsize )) !== FALSE) {
 fwrite( $outfile, $str );
 if ( $showprogress ) {
 $progress->downloaded( strlen( $str ) );
 }
 }
 
 fclose( $handle );
 fclose( $outfile );

 if( $showprogress ) {
 $progress->done();
 }

 return BundleBuild::parseBundleMetaInfoFromZipFile($filename);
 }

 function &parseBundleMetaInfoFromZipFile( $zipfile ) {
 $filename = $zipfile;
 $dir = tempdir( '/Node/tmp' );
 $xml = array();
 $cmd = "cd $dir && /Node/java/bin/jar xf $filename bundlebuild.xml && cat bundlebuild.xml && rm -rf $dir";
 
 exec( $cmd, $xml );
 $xml = implode("\n", $xml);
 
 
 $bundle = Bundle::parseXML($xml, false);
 return $bundle;
 }

 



 function parseObjectInfoFromZipFile( $tables, $zipfile ) {
 $filename = $zipfile;
 $dir = tempdir( '/Node/tmp' );

 foreach( $tables as $table ) {
 $ttype = $table['ttype'];
 
 if( !$ttype || $table['count'] == 0 ) {
 continue;
 }

 
 
 if ($this->_objects[$ttype]) {
 continue;
 }

 
 $xmlfile_name = $table['name'] . "_objects.xml";
 $xml = array();
 $cmd = "cd $dir && /Node/java/bin/jar xf $filename $xmlfile_name && cat $xmlfile_name";
 exec( $cmd, $xml, &$retval );

 $xml = implode("\n", $xml);
 $this->parseObjectsInformation($ttype, &$xml);
 }

 
 
 }

 

 
 function parseObjectsInformation($ttype, &$xml) {
 $vals = null;
 $index = null;
 
 
 
 
 
 
 
 
 $p = xml_parser_create();
 if (!xml_parse_into_struct($p, $xml, $vals, $index)) {
 return NULL;
 }
 xml_parser_free($p);
 $this->_objects[$ttype] = array();
 $currentObj = null;
 
 $numvals = count($vals);
 for ($i = 0 ; $i < $numvals ; $i++ ) {
 $tagname = strtolower($vals[$i]['tag']);
 switch ($vals[$i]['type']) {
 case 'open':
 if ($tagname == 'object') {
 
 if ($vals[$i]['attributes']['EXCLUDED'] != 'true') {
 $currentObj = array('name' => $vals[$i]['attributes']['NAME'] ? $vals[$i]['attributes']['NAME'] : i18n('import_list_name_unknown'), 'ttype' => $ttype);
 }
 }
 break;
 case 'close':
 if ($tagname == 'object') {
 if ($currentObj) {
 $this->_objects[$ttype][] = $currentObj;
 $currentObj = null;
 }
 }
 break;
 case 'complete':
 if ($tagname == 'globalid' && $currentObj) {
 $currentObj['globalid'] = $vals[$i]['value'];
 }
 break;
 }
 }

 
 usort(&$this->_objects[$ttype], 'sort_by_name_function');
 }

 function startExport($dryrun = false) {
 $sql = "UPDATE bundlebuild SET statuscode = -1, filename = NULL WHERE id = '".sqlf_escaped($this->getID())."'";
 query( $sql );
 node_class("codeparser");
 
 $javaParser =& CodeParser::getJavaParser();
 return $javaParser->startBundleExport( $this->getID(), $dryrun );
 }

 function cancelExport() {
 global $USER_ID, $FEATURE;
 if (perm( $USER_ID, T_BUNDLE_EXPORT_ELEMENT, $this->getFieldValue("bundle_id"), 2 ) || perm( $USER_ID, T_BUNDLE, '', 5 )) {
 node_class("codeparser");
 $javaParser =& CodeParser::getJavaParser();
 $javaParser->cancelBundle( $FEATURE["export2"] ? 'export2' : 'export', $this->getID() );
 }
 }

 function getDate() {
 return $this->getFieldValue("date");
 }

 function &createNewImport($startdryrun = true, $group_id = -1) {
 global $USER_ID;
 
 $bundleid = $this->getFieldValue("bundle_id");
 $sql = "DELETE FROM bundleimport WHERE bundlebuild_id IN (SELECT id FROM bundlebuild WHERE bundle_id='".sqlf_escaped($bundleid)."') AND date = 0";
 query( $sql );
 $bundleimport = new BundleImport();
 $bundleimport->setFieldValue('bundlebuild_id', $this->getID());
 $bundleimport->setFieldValue('user_id', $USER_ID);
 $bundleimport->setFieldValue('group_id', $group_id);
 $bundleimport->save();
 if( $startdryrun === true ) {
 $bundleimport->startImportDryrun();
 }
 return $bundleimport;
 }

 function getFileSize() {
 $filename = $this->getBundlePath().$this->getFieldValue('filename');
 $fsize = filesize($filename);
 return $fsize;
 }

 function handleDownload() {
 global $FILEPATH;
 $filename = $this->getBundlePath().$this->getFieldValue('filename');
 $fsize = filesize($filename);
 
 header('Content-Type: application/zip');
 header("Accept-Range: bytes"); 
 header("Content-Length: $fsize");
 header("Content-Disposition: attachment; filename=\"".$this->getFieldValue('filename')."\";");

 
 global $FEATURE, $GZCOMPRESS, $STAG_PREFIX;
 $FEATURE['use_default_gzcompress'] = false;
 $old = $GZCOMPRESS;
 $GZCOMPRESS = false;
 ob_end_flush();
 $GZCOMPRESS = $old;
 
 
 
 ob_end_flush();
 $file = fopen( $filename, 'rb' );
 fpassthru($file);
 fclose($file);
 ob_start();
 }

 function onBeforeDeleteObject() {
 
 $objs = &$this->getContainedObjects(false);
 foreach( $objs as $obj ) {
 $obj->delete();
 }

 
 $sql = "SELECT id FROM bundleimport WHERE bundlebuild_id=".sqlf_tl_num($this->getID());
 $rs = query($sql);
 while(list($key, $val) = each($rs["arr"])) {
 $bundleImport = &NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORT, $val["id"] ); 
 $bundleImport->delete();
 }

 
 $filename = $this->getFieldValue('filename');
 if($filename) {
 unlink( $this->getBundlePath().$filename );
 }
 }

 function onInsertObject() {
 
 $bundle =& $this->getBundle();
 $keepArchive = $bundle->getFieldValue("keepoldbuilds");
 if (! $keepArchive) {
 $builds =& $bundle->getBuilds();
 foreach($builds as $build) {
 if ($build->getID() != $this->getID()) {
 $build->deleteBuildFile();
 }
 }
 }
 }

 function getContainedObjectCount() {
 $sql = "SELECT COUNT(*) as count FROM bundlecontainedobject WHERE bundle_id = '" . sqlf_escaped($this->getFieldValue('bundle_id')) . "' AND bundlebuild_id = '".sqlf_escaped($this->getID())."' AND referrer_obj_id=0 AND referrer_obj_type=0 AND excluded = 0 AND channel_id = 0";
 $rs = query( $sql );
 if (count($rs["arr"]) < 1) {
 return 0;
 }
 return $rs["arr"][0]["count"];
 }

 function &getLatestBundleImport() {
 $sql = "SELECT bundleimport.id AS id FROM bundleimport JOIN (bundlebuild) ON (bundleimport.bundlebuild_id=bundlebuild.id) WHERE bundlebuild.id = '" . sqlf_escaped($this->getID()) . "' ORDER BY bundleimport.date DESC LIMIT 1";
 $rs = query( $sql );
 if (count($rs["arr"]) < 1) {
 return null;
 }
 return NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORT, $rs["arr"][0]["id"] );
 }

 function deleteBuildFile() {
 
 $filename = $this->getFieldValue('filename');
 if($filename) {
 unlink( $this->getBundlePath().$filename );
 }
 
 
 }

 function createObjects($ttype, &$arr) {
 $ret = array( );
 foreach( $arr as $row ) {

 
 $obj = NodeFactory::getStaticInstanceOf( $ttype, $row['id'] );
 if( !$obj->isInitialized() ) {
 $obj->initFromResultSet( $row, '', true );
 }
 $ret[] = $obj;
 }
 return $ret;
 }

 




 function &createCopy($new_bundle_id = null) {
 $new_build = new BundleBuild();
 if (!$new_bundle_id) {
 $new_bundle_id = $this->getFieldValue("bundle_id");
 }
 $new_build->setFieldValue("bundle_id", $new_bundle_id);
 $new_build->setFieldValue("changelog", $this->getFieldValue("changelog"));
 $new_build->setFieldValue('statuscode', 4);
 $new_build->setFieldValue('date', time());
 $new_build->save();

 $sql = "INSERT INTO bundlecontainedobject (bundle_id, obj_type, obj_id, channel_id, referrer_obj_id, accepted, referrer_obj_type, autoadded, cause_obj_type, cause_obj_id, excluded, bundlebuild_id) SELECT ".sqlf_tl_num($new_bundle_id).", obj_type, obj_id, channel_id, referrer_obj_id, accepted, referrer_obj_type, autoadded, cause_obj_type, cause_obj_id, excluded, ".sqlf_tl_num($new_build->getID())." FROM bundlecontainedobject WHERE bundlebuild_id = ".sqlf_tl_num($this->getID());
 $rs = query($sql);

 return $new_build;
 }

 




 function getContainedObjects($onlyselected = true, $omitinherited = false) {
 $sql = "SELECT * FROM bundlecontainedobject WHERE bundle_id = '".sqlf_escaped($this->getFieldValue('bundle_id'))."' AND bundlebuild_id = '".sqlf_escaped($this->getID())."' AND referrer_obj_id = 0 and referrer_obj_type = 0 and excluded = 0";
 if( $onlyselected ) {
 $sql .= " and autoadded = 0 ";
 }
 if ($omitinherited) {
 $sql .= " AND channel_id = 0";
 }
 $sql .= " ORDER BY obj_type ";
 $rs = query( $sql );
 return $this->createObjects( T_BUNDLE_CONTAINED_OBJECT, $rs['arr'] );
 }

 function getExcludedObjects() {
 $sql = "SELECT * FROM bundlecontainedobject WHERE bundle_id = '".sqlf_escaped($this->getFieldValue('bundle_id'))."' AND bundlebuild_id = '".sqlf_escaped($this->getID())."' AND excluded = 1 ORDER BY obj_type";
 $rs = query( $sql );
 return $this->createObjects( T_BUNDLE_CONTAINED_OBJECT, $rs['arr'] );
 }

 function getSubselectedFolders() {
 $bundleId = $this->getFieldValue('bundle_id');
 $bundleBuildId = $this->getID();
 

 
 $sql = "select folder_id, count(*) c from bundlecontainedobject left join page on bundlecontainedobject.obj_id = page.id where bundle_id = ".sqlf_tl_num($bundleId)." and bundlebuild_id = ".sqlf_tl_num($bundleBuildId)." and obj_type = 10007 AND referrer_obj_id = 0 and referrer_obj_type = 0 and excluded = 0 group by folder_id";
 $rs1 = query($sql);

 
 $sql ="select folder_id, count(*) c from bundlecontainedobject left join contentfile on bundlecontainedobject.obj_id = contentfile.id where bundle_id = ".sqlf_tl_num($bundleId)." and bundlebuild_id = ".sqlf_tl_num($bundleBuildId)." and obj_type in(10008, 10011) AND referrer_obj_id = 0 and referrer_obj_type = 0 and excluded = 0 group by folder_id";
 $rs2 = query($sql);

 
 $sql = "select mother folder_id, count(*) c from bundlecontainedobject left join folder on bundlecontainedobject.obj_id = folder.id where bundle_id = ".sqlf_tl_num($bundleId)." and bundlebuild_id = ".sqlf_tl_num($bundleBuildId)." and obj_type = 10002 AND referrer_obj_id = 0 and referrer_obj_type = 0 and excluded = 0 group by folder_id";
 $rs3 = query($sql);

 return array_merge($rs1, $rs2, $rs3);
 }
 





 function setContainedObjects( $containedObjects, $excludedObjects ) {
 global $T_DOMAIN_CHANNEL;

 $sql = "DELETE FROM bundlecontainedobject WHERE bundle_id = '".sqlf_escaped($this->getFieldValue('bundle_id'))."' AND bundlebuild_id = '".sqlf_escaped($this->getID())."' AND referrer_obj_id = 0";
 query($sql);

 if( is_a($containedObjects, "ObjectSelectionStore")) {
 
 while( list($type, $ids) = each($containedObjects->objectStore)) {
 while (list($id, $v) = each($ids) ) {
 $id = explode("_", $id);
 $o = new BundleContainedObject();
 $o->setFieldValue( 'bundle_id', $this->getFieldValue('bundle_id') );
 $o->setFieldValue( 'bundlebuild_id', $this->getID() );
 $o->setFieldValue( 'obj_type', $type );
 $o->setFieldValue( 'obj_id', $id[0] );
 
 if ($type == $T_DOMAIN_CHANNEL) {
 $folderinfo = get_folderinfo($id[0]);
 $o->setFieldValue( 'channel_id', $folderinfo["node_id"]);
 } else {
 $o->setFieldValue( 'channel_id', $id[1] );
 }
 $o->setFieldValue( 'referrer_obj_id', 0 );
 $o->setFieldValue( 'referrer_obj_type', 0 );
 $o->save();
 }
 }

 
 while( list($type, $v) = each($containedObjects->typeStore)) {
 $o = new BundleContainedObject();
 $o->setFieldValue( 'bundle_id', $this->getFieldValue('bundle_id') );
 $o->setFieldValue( 'bundlebuild_id', $this->getID() );
 $o->setFieldValue( 'obj_type', $type );
 $o->setFieldValue( 'obj_id', 0 );
 $o->setFieldValue( 'referrer_obj_id', 0 );
 $o->setFieldValue( 'referrer_obj_type', 0 );
 $o->save();
 }
 }

 
 if (is_a($excludedObjects, "ObjectSelectionStore")) {
 
 while( list($type, $ids) = each($excludedObjects->objectStore)) {
 while (list($id, $v) = each($ids) ) {
 $id = explode("_", $id);
 $o = new BundleContainedObject();
 $o->setFieldValue( 'bundle_id', $this->getFieldValue('bundle_id') );
 $o->setFieldValue( 'bundlebuild_id', $this->getID() );
 $o->setFieldValue( 'obj_type', $type );
 $o->setFieldValue( 'obj_id', $id[0] );
 $o->setFieldValue( 'channel_id', $id[1] );
 $o->setFieldValue( 'referrer_obj_id', 0 );
 $o->setFieldValue( 'referrer_obj_type', 0 );
 $o->setFieldValue( 'excluded', 1);
 $o->save();
 }
 }

 
 while( list($type, $v) = each($excludedObjects->typeStore)) {
 $o = new BundleContainedObject();
 $o->setFieldValue( 'bundle_id', $this->getFieldValue('bundle_id') );
 $o->setFieldValue( 'bundlebuild_id', $this->getID() );
 $o->setFieldValue( 'obj_type', $type );
 $o->setFieldValue( 'obj_id', 0 );
 $o->setFieldValue( 'referrer_obj_id', 0 );
 $o->setFieldValue( 'referrer_obj_type', 0 );
 $o->setFieldValue( 'excluded', 1);
 $o->save();
 }
 }

 
 $sql = "UPDATE bundlebuild SET statuscode = 6 WHERE id = ".sqlf_tl_num($this->getID());
 query($sql);
 $this->setFieldValue('statuscode', 6);
 }

 



 function isEditable() {
 if ($this->isNew()) {
 return true;
 }
 switch($this->getFieldValue("statuscode")) {
 case 4:
 case 5:
 case 6:
 $sql = "SELECT id FROM bundlebuild WHERE bundle_id = '".sqlf_escaped($this->getFieldValue("bundle_id"))."' ORDER BY date DESC LIMIT 1";
 $rs = query($sql);
 return $rs["arr"][0]["id"] == $this->getID();
 default:
 return false;
 }
 }

 




 function getContainedObjectsOverview($table = false) {
 global $T_TYPE, $ICO_ALERT_ANI;
 $sql = "SELECT count(id) as c, obj_type FROM bundlecontainedobject WHERE bundle_id = '".sqlf_escaped($this->getFieldValue('bundle_id'))."' AND bundlebuild_id = '".sqlf_escaped($this->getID())."' AND referrer_obj_id = 0 and excluded = 0 AND channel_id = 0 GROUP by obj_type";
 $rs = query( $sql );
 $contained = '';
 if ($table) {
 $contained .= '<table border="0" cellspacing="0" cellpadding="1">';
 } else {
 $contained .= '<ul id="exportdetails">';
 }
 $foundsomething = false;
 foreach( $rs['arr'] as $row ) {
 if ($table) {
 $contained .= "<tr><td style=\"text-align:right\">".$row['c'] . 'x ' ."</td><td>" . img( $T_TYPE[$row['obj_type']]['folderclosed'], 0, '', '', '', '', '', '', '', 'vertical-align:text-bottom;' ) . '</td><td>' . $T_TYPE[$row['obj_type']]['name'] . '</td>\n';
 } else {
 $contained .= '<li style="background-image: url('.$T_TYPE[$row['obj_type']]['folderclosed'].')">'.$row['c'].'x '.$T_TYPE[$row['obj_type']]['name'].'</li>';
 }
 $foundsomething = true;
 }
 if (!$foundsomething) {
 if ($table) {
 $contained .= "<tr><td>".i18n("importexport_empty")."</td></tr>";
 } else {
 $contained .= i18n("importexport_empty")."<br/>";
 }
 }

 if ($table) {
 $contained .= '</table>';
 } else {
 $contained .= '</ul>';
 }
 return $contained;
 }
}

class BundleImport extends NodeObject {
 var $bundleimport_id;

 function _getNodeTable() {
 return "bundleimport";
 }

 function getValueFields() {
 return array( 'id' => false, 'user_id' => 'int', 'date' => 'int', 'bundlebuild_id' => 'int', 'group_id' => 'int' );
 }

 function _setID($id) {
 $this->bundleimport_id = $id;
 }

 function getID() {
 return $this->bundleimport_id;
 }

 function cancelImport() {
 
 global $USER_ID, $FEATURE;
 $bundleBuild = &NodeFactory::getStaticInstanceOf(T_BUNDLE_BUILD, $this->getBundleBuildId());
 $bundle = &$bundleBuild->getBundle();
 if (perm( $USER_ID, T_BUNDLE_IMPORT_ELEMENT, $bundle->getID(), 2 ) || perm( $USER_ID, T_BUNDLE, '', 6 )) {
 node_class("codeparser");
 $javaParser =& CodeParser::getJavaParser();
 $javaParser->cancelBundle( $FEATURE['import2'] ? 'import2' : 'import', $this->getID() );
 }
 }

 function getStatus() {
 global $STAG, $FEATURE;
 $sql = "SELECT bundleimport.date AS importdate, bundlebuild.date AS builddate, bundlebuild.statuscode AS statuscode FROM bundleimport JOIN (bundlebuild) ON (bundleimport.bundlebuild_id=bundlebuild.id) WHERE bundleimport.id='" . sqlf_escaped($this->getID()) . "' ORDER BY importdate != 0, importdate DESC;";
 $rs = query( $sql );
 if (count($rs["arr"]) < 1) {
 return array( "message" => i18n("Undefined") );
 }
 $statuscode = $rs["arr"][0]["statuscode"];
 $importdate = $rs["arr"][0]["importdate"];
 $builddate = $rs["arr"][0]["builddate"];
 if ($statuscode == -1) {
 node_class("codeparser");
 $javaParser =& CodeParser::getJavaParser();
 $status = $javaParser->getQueueStatus( $FEATURE['import2'] ? 'import2' : 'import', $this->getID() );
 $s = $status['status'];
 
 if( $s == 'QUEUED' ) {
 return array( "message" => i18n( 'Queued {0} Position', 0, $status['queuePosition'] ), "status" => 'QUEUED ' . $status['queuePosition'] );
 } else if( $s == 'INPROGRESS' ) {
 return array( "message" => i18n( 'In Progress ({0} %)', 0, (($status['progressPercent'] >= 0) ? $status['progressPercent'] : 0 ) ), "status" => 'QUEUED ' . $status['progressPercent'], "cancel" => "<a href=\"$STAG&do=1010&cmd=cancel&type=import&bundleimportid=" . $this->getID() . "\">" . i18n("Cancel") . "</a>" );
 }
 
 return array( "message" => i18n("Internal error"), "status" => 'ERROR' );
 
 }
 if ($statuscode == 3) {
 return array( "message" => i18n("sn_process_canceled"), "error" => true, "status" => "ERROR" );
 }
 if ($statuscode > 0) {
 return array( "message" => i18n("Internal error"), "error" => true, 'status' => 'ERROR' );
 }
 if ($importdate > 0) {
 return array( "message" => i18n("Successful Import on {0}", 0, date("Y-m-d H:i", $importdate)), 'status' => 'DONE' );
 }
 
 $sql = "SELECT COUNT(bundleimportconflict.id) AS conflicts FROM bundleimportconflict LEFT JOIN (bundleimportobject) ON (bundleimportconflict.bundleimportobject_id=bundleimportobject.id) WHERE bundleimportobject.bundleimport_id = '" . sqlf_escaped($this->getID()) . "';";
 $rs = query( $sql );
 $url = "$STAG&do=1019&bundleimportid=" . $this->getID();
 if (count($rs["arr"]) < 1 || $rs["arr"][0]["conflicts"] == 0) {
 return array( "message" => i18n("Dryrun finished. No conflicts."), "link" => $url, 'status' => 'DRYRUNFINISHED' );
 }
 return array( "message" => i18n("Dryrun finished. Conflicts exist."), "link" => $url, "error" => true, 'status' => 'DRYRUNFINISHEDWITHCONFLICTS' );
 }

 function getConflicts() {
 $sql = "SELECT bundleimportobject.globalprefix, bundleimportobject.globalid, bundleimportconflict.id AS id, ".
 "bundleimportconflict.name AS name, bundleimportconflict.reason AS reason, bundleimportobject.obj_type AS obj_type, ".
 "bundleimportconflict.reason_obj_type AS reason_obj_type, bundleimportconflict.reason_obj_id AS reason_obj_id, ".
 "bundleimportobject.action as action, bundleimportconflict.conflict_behaviour, bundleimportconflict.recoverable ".
 "FROM bundleimportconflict JOIN (bundleimportobject) ON (bundleimportconflict.bundleimportobject_id=bundleimportobject.id) ".
 "WHERE bundleimportobject.bundleimport_id='" . sqlf_escaped($this->getID()) . "' ORDER BY bundleimportobject.obj_type ASC, name ASC, reason_obj_id ASC";
 $rs = query( $sql );
 $ret = array();
 foreach( $rs["arr"] as $row ) {
 $obj = NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORTCONFLICT, $row['id'] );
 if( !$obj->isInitialized() ) {
 $obj->initFromResultSet( $row, '', true );
 }
 $ret[] = $obj;
 }
 
 return $ret;
 }

 function hasUnrecoverableConflicts() {
 $sql = "SELECT count(*) c FROM bundleimportconflict JOIN bundleimportobject ON (bundleimportconflict.bundleimportobject_id=bundleimportobject.id) WHERE bundleimportobject.bundleimport_id = '".sqlf_escaped($this->getID())."' AND recoverable = 0";
 $rs = query( $sql );
 return $rs["arr"][0]["c"] > 0;
 }

 function hasMissingFolderConflicts() {
 $sql = "SELECT count(*) c FROM bundleimportconflict JOIN bundleimportobject ON (bundleimportconflict.bundleimportobject_id=bundleimportobject.id) WHERE bundleimportobject.bundleimport_id = '".sqlf_escaped($this->getID())."' AND reason = 'folder'";
 $rs = query( $sql );
 return $rs["arr"][0]["c"] > 0;
 }

 function getSummary($default_behaviour) {
 global $T_TYPE;
 $sql = "SELECT
        COUNT(DISTINCT bundleimportobject.id) AS count,
        bundleimportobject.action AS action,
        bundleimportobject.obj_type AS obj_type,
        bundleimportconflict.id IS NOT NULL AS isconflict,
        IF(conflict_behaviour IS NULL, ".sqlf_tl_num($default_behaviour).", conflict_behaviour) AS behaviour
      FROM bundleimportobject
        LEFT JOIN (bundleimportconflict) ON (bundleimportconflict.bundleimportobject_id=bundleimportobject.id AND bundleimportconflict.reason != 'folder')
      WHERE bundleimportobject.bundleimport_id='" . sqlf_escaped($this->getID()) . "' and (action is null || action != 'excluded')
      GROUP BY action, behaviour, obj_type, isconflict
      ORDER BY obj_type, isconflict";
 $rs = query( $sql );
 $summary = array();
 foreach ($rs['arr'] as $row) {
 $icon = $T_TYPE[$row["obj_type"]]["folderclosed"];
 $typename = $T_TYPE[$row["obj_type"]]["name"];
 $entry = array( "count" => $row["count"], "action" => $row["action"], "type" => $typename,
 "isconflict" => $row["isconflict"], "icon" => $icon, "conflict_behaviour" => $row["behaviour"] );
 $summary[] = $entry;
 }
 return $summary;
 }

 function onBeforeInsertObject() {
 
 if( !$this->getFieldValue('date') ) {
 $sql = "DELETE FROM bundleimport WHERE bundlebuild_id = '" . sqlf_escaped($this->getFieldValue('bundlebuild_id')) . "' AND (date IS NULL OR date = 0)";
 query( $sql );
 }
 }

 function startImportDryrun($importfolder_id = false) {
 $this->startImport(true, "", $importfolder_id);
 }

 function startImport($dryrun = false, $conflictBehavior = "", $importfolder_id = false) {
 $sql = "UPDATE bundlebuild SET statuscode = -1 WHERE id = '".sqlf_escaped($this->getFieldValue('bundlebuild_id')) . "'";
 query($sql);

 if ($importfolder_id) {
 
 $sql = "UPDATE bundle SET importfolder_id = '".sqlf_escaped($importfolder_id)."' WHERE id = (SELECT bundle_id FROM bundlebuild WHERE id = '" . sqlf_escaped($this->getFieldValue("bundlebuild_id")) . "')";
 query ($sql);
 } else {
 
 $sql = "SELECT bundle.importfolder_id FROM bundlebuild INNER JOIN bundle ON bundle.id = bundlebuild.bundle_id WHERE bundlebuild.id = '" . sqlf_escaped($this->getFieldValue("bundlebuild_id")) . "'";
 $rs = query( $sql );
 $importfolder_id = $rs['arr'][0]['importfolder_id'];
 }
 node_class("codeparser");
 $javaParser =& CodeParser::getJavaParser();
 $javaParser->startBundleImport( $this->getID(), $dryrun, $importfolder_id, $conflictBehavior, $this->getFieldValue("group_id") );
 }

 function getBundleBuildId() {
 return $this->getFieldValue("bundlebuild_id");
 }

 function onBeforeDeleteObject() {
 
 $sql = "SELECT id FROM bundleimportobject WHERE bundleimport_id=".sqlf_tl_num($this->getID());
 $rs = query($sql);
 while(list($key, $val) = each($rs["arr"])) {
 $bundleImportObj = &NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORT_OBJECT, $val["id"] );
 $bundleImportObj->delete();
 }
 }

 function getContainedObjects($omitignored = false) {
 $sql = "SELECT bundleimportobject.id as id, obj_type, action, localid, bundleimportobject.globalprefix, bundleimportobject.globalid FROM bundleimportobject JOIN (mappedglobalid) ON (bundleimportobject.globalprefix=mappedglobalid.globalprefix AND bundleimportobject.globalid=mappedglobalid.globalid) WHERE bundleimport_id='".sqlf_escaped($this->getID())."' AND localid != 0 and action != 'excluded'";
 if ($omitignored) {
 $sql .= " AND action != 'ignored'";
 }
 $rs = query($sql);
 $ret = array();
 foreach ($rs["arr"] as $row) {
 $bundleimportobject = NodeFactory::getStaticInstanceOf( $row["obj_type"], $row["localid"] );
 $bundleimportobject->setFieldValue("action", $row["action"]);
 $bundleimportobject->setFieldValue("globalprefix", $row["globalprefix"]);
 $bundleimportobject->setFieldValue("globalid", $row["globalid"]);
 $ret[] = $bundleimportobject;
 }
 return $ret;
 }

 function getDeletedObjects() {
 global $T_TYPE;
 $sql = "SELECT count(*) AS count, obj_type FROM bundleimportobject WHERE bundleimport_id='" . sqlf_escaped($this->getID()) . "' AND action='deleted' GROUP BY obj_type ORDER BY obj_type";
 $rs = query($sql);
 $ret = array();
 foreach ($rs["arr"] as $row) {
 $ret[] = array("icon" => $T_TYPE[$row["obj_type"]]["folderclosed"], "count" => $row["count"], "name" => $T_TYPE[$row["obj_type"]]["name"]);
 }
 return $ret;
 }
}

class BundleImportConflict extends NodeObject {
 var $id;

 function _getNodeTable() {
 return "bundleimportconflict";
 }

 function getValueFields() {
 return array( 'id' => false, 'name' => 'text', 'reason' => 'text', 'obj_type' => 'int',
 'reason_obj_type' => 'int', 'reason_obj_id' => 'int',
 'bundleimportobject_id' => 'int', 'action' => 'text', 'conflict_behaviour' => 'text', 'recoverable' => 'int',
 'globalprefix' => 'text', 'globalid' => 'int' );
 }

 function _setID($id) {
 $this->id = $id;
 }

 function getID() {
 return $this->id;
 }

 function getTypeInBundle() {
 global $T_TYPE;
 return "<img src=\"" . $T_TYPE[$this->getFieldValue("obj_type")]["folderclosed"] . "\" style=\"vertical-align:text-bottom;\"> "
 . $T_TYPE[$this->getFieldValue("obj_type")]["name"];
 }

 function getNameInBundle() {
 return $this->getFieldValue("name");
 }

 function getNameInSystem() {
 $ttype = $this->getFieldValue("reason_obj_type");
 $id = $this->getFieldValue("reason_obj_id");
 if (!$ttype || !$id) {
 return "";
 }
 $object =& NodeFactory::getStaticInstanceOf( $ttype, $id );
 return $object->getDisplayName();
 }

 function getTypeInSystem() {
 global $T_TYPE;
 return "<img src='" . $T_TYPE[$this->getFieldValue("reason_obj_type")]["folderclosed"] . "' border='0' style='vertical-align:text-bottom;'> "
 . $T_TYPE[$this->getFieldValue("reason_obj_type")]["name"];
 }

 function getDisplayURL() {
 $ttype = $this->getFieldValue("reason_obj_type");
 $id = $this->getFieldValue("reason_obj_id");
 $object =& NodeFactory::getStaticInstanceOf( $ttype, $id );
 if(method_exists($object, "getDisplayURL")) {
 $url = $object->getDisplayURL();
 if ($url) {
 return $url;
 }
 }
 return null;
 }

 function getReason() {
 
 
 
 
 return i18n( 'conflict_reason_'.$this->getFieldValue("reason") );
 }

 function getImportObjectId() {
 return $this->getFieldValue("bundleimportobject_id");
 }

 function getAction() {
 return $this->getFieldValue('action');
 }
}


function tempdir($dir, $prefix='', $mode=0700)
{
 if (substr($dir, -1) != '/') $dir .= '/';

 do
 {
 $path = $dir.$prefix.mt_rand(0, 9999999);
 } while (!mkdir($path, $mode));

 return $path;
}

class BundleImportObject extends NodeObject {
 var $id;

 function _getNodeTable() {
 return "bundleimportobject";
 }

 function getValueFields() {
 return array( 'bundleimport_id' => 'int', 'action' => 'text', 'update' => 'int', 'globalprefix' => 'text', 'globalid' => 'int', 'obj_type' => 'int');
 }

 function _setID($id) {
 $this->id = $id;
 }

 function getID() {
 return $this->id;
 }

 function onBeforeDeleteObject() {
 
 $sql = "SELECT id FROM bundleimportconflict WHERE bundleimportobject_id=".sqlf_tl_num($this->getID());
 $rs = query($sql);
 while(list($key, $val) = each($rs["arr"])) {
 $bundleImportConflict = &NodeFactory::getStaticInstanceOf( T_BUNDLE_IMPORTCONFLICT, $val["id"] );
 $bundleImportConflict->delete();
 }
 }
}














class StatusFormatter extends ResultsetFormatter {
 function format( $rs ) {
 $bundle = &NodeFactory::getStaticInstanceOf(T_BUNDLE, $rs['id']);
 $s = $bundle->getStatusMessage();
 checkreload($s);
 if($s['html']) {
 return $s['html'];
 }
 return $s['message'];
 
 }
}

class ImportStatusFormatter extends ResultsetFormatter { 
 function format( $rs ) {
 global $USER_ID;
 $id = $rs["id"];
 $bundle = &NodeFactory::getStaticInstanceOf(T_BUNDLE, $id);
 $bundlebuild = $bundle->getLatestBundleImport();
 $status = $bundlebuild->getStatus();
 checkreload($status);
 $style = "";
 if ($status["error"]) {
 $style = " style=\"color: #FF8827;\" ";
 }
 if ($status["link"] &&
 (perm( $USER_ID, T_BUNDLE_IMPORT_ELEMENT, $id, 2 ) || perm( $USER_ID, T_BUNDLE, '', 6 ))) {
 return "<a $style href=\"" . $status["link"] . "\">" . $status["message"] . "</a>";
 }
 if ($status["cancel"] &&
 (perm( $USER_ID, T_BUNDLE_IMPORT_ELEMENT, $id, 2 ) || perm( $USER_ID, T_BUNDLE, '', 6 ))) {
 return $status["message"] . " / " . $status["cancel"];
 }
 return $status["message"];
 }
}

class InstalledVersionFormatter extends ResultsetFormatter {
 function format( $rs ) {
 $id = $rs["id"];
 $bundle = &NodeFactory::getStaticInstanceOf(T_BUNDLE, $id);
 $bundlebuild = &$bundle->getInstalledBundleBuild();
 $date = i18n("not installed");
 if ($bundlebuild) {
 $date = date("Y-m-d H:i", $bundlebuild->getDate());
 
 }
 return $date;
 }
}

class ImportTypeFormatter extends ResultsetFormatter {
 function format( $rs ) {
 return $rs["updateurl"] ? i18n("URL") : i18n("file");
 }
}

class AvailableVersionFormatter extends ResultsetFormatter {
 function format( $rs ) {
 global $STAG, $DO, $USER_ID;
 $id = $rs["id"];
 $bundle = &NodeFactory::getStaticInstanceOf(T_BUNDLE, $id);
 $latestBuild = &$bundle->getLatestBuild();
 $latestBundleImport = &$bundle->getLatestBundleImport();
 $date = "non available";
 if ($latestBuild) {
 $date = date("Y-m-d H:i", $latestBuild->getDate());
 }
 if ($latestBuild->getID() != $latestBundleImport->getBundleBuildId() &&
 (perm( $USER_ID, T_BUNDLE_IMPORT_ELEMENT, $id, 2 ) || perm( $USER_ID, T_BUNDLE, '', 6 ))) {
 $date .= " / <a href=\"$STAG&do=$DO&cmd=startdryrun&bundlebuild_id=".$latestBuild->getID()."\">".i18n("Install Update")."</a>";
 }
 return $date;
 }
}


class SingleDeleteActionFormatter extends ResultsetFormatter {
 function format( $rs ) {
 global $STAG, $ICO_DELETE;
 $id = $rs['id'];
 return "&nbsp;&nbsp;<a href=\"$STAG&do=1017&BUNDLE_ID=$id\"><img src=\"$ICO_DELETE\" border=\"0\" alt=\"" . i18n("Delete") . "\" title=\"" . i18n("Delete") . "\"></a>";
 }
}


class BundlePermissionFilter extends SQLDatasourceFilter {
 function checkRow( $row ) {
 global $USER_ID;
 $ttype = $row['isimported'] ? T_BUNDLE_IMPORT_ELEMENT : T_BUNDLE_EXPORT_ELEMENT;
 return perm( $USER_ID, $ttype, $row['id'], 0 ) && perm( $USER_ID, T_BUNDLE, '', 0);
 }
}

class NameEscapeFormatter extends ResultsetFormatter {
 function format( $rs ) {
 return htmlentities_utf8($rs["name"]);
 }
}


?>
